<?php

namespace Drupal\state_log;

use Drupal\Core\State\StateInterface;
use Psr\Log\LoggerInterface;

/**
 * Logs changes to state.
 */
class StateLog implements StateInterface {

  /**
   * Decorated service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $decorated;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new StateLog.
   *
   * @param \Drupal\Core\State\StateInterface $decorated
   *   The decorated service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(StateInterface $decorated, LoggerInterface $logger) {
    $this->decorated = $decorated;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function get($key, $default = NULL) {
    return $this->decorated->get($key, $default);
  }

  /**
   * {@inheritdoc}
   */
  public function getMultiple(array $keys) {
    return $this->decorated->getMultiple($keys);
  }

  /**
   * {@inheritdoc}
   */
  public function set($key, $value) {
    $this->log($key, $value);
    $this->decorated->set($key, $value);
  }

  /**
   * {@inheritdoc}
   */
  public function setMultiple(array $data) {
    foreach ($data as $key => $value) {
      $this->log($key, $value);
    }
    $this->decorated->setMultiple($data);
  }

  /**
   * {@inheritdoc}
   */
  public function delete($key) {
    $this->log($key, NULL);
    $this->decorated->delete($key);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteMultiple(array $keys) {
    foreach ($keys as $key) {
      $this->log($key, NULL);
    }
    $this->decorated->deleteMultiple($keys);
  }

  /**
   * {@inheritdoc}
   */
  public function resetCache() {
    $this->decorated->resetCache();
  }

  /**
   * Returns whether the state_name is ignored.
   *
   * @param string $state_name
   *   The state name key.
   *
   * @return bool
   *   TRUE if the state key is ignored, FALSE otherwise.
   */
  private function isKeyIgnored($state_name) {
    $ignored_entities = \Drupal::config('state_log.settings')->get('log_ignored_state') ?: [];
    foreach ($ignored_entities as $ignore) {
      if ($ignore == $state_name) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Logs the change to the state data.
   *
   * @param string $key
   *   The key of the data being logged.
   * @param mixed $value
   *   The data being logged.
   */
  protected function log($key, $value) {
    if (!$this->isKeyIgnored($key)) {
      $this->logger->info('State changed: %key changed from %original to %value', [
        '%key' => $key,
        '%original' => var_export($this->get($key), TRUE),
        '%value' => var_export($value, TRUE),
      ]);
    }
  }

}
